require "gemoji"

class Content::Pipeline::Filters::Gemoji < Content::Pipeline::Filter
  EmojiTag = %Q{<img class="emoji" src="%s" alt=":%s:" height="20" width="20">}
  EmojiPattern = /(?:^\s*|\s+):(#{Emoji.all.map { |n| Regexp.escape(n.name) }.join("|")}):(?:\s+|\s*$)/m
  EmojiLiquidTag = %Q{{%%img "%s" ":%s:" %%}}

  add_filter({
    :gemoji => :nokogiri
  })

  def initialize(*args)
    super(*args)
    @opts[:asset_path] ||= \
      "/assets"
  end

  private
  def gemoji
    @str = @str.to_nokogiri_fragment
    @str.xpath(".//child::text()").each do |n|
      parse_node(
        n
      )
    end
  end

  def parse_node(node)
    return node if node.ancestors.any? do |n|
      ["pre", "code"].include?(
        n.node_name
      )
    end

    node.replace(node.to_html.gsub(EmojiPattern) do
      ep = "#{@opts[:asset_path].chomp("/")}/#{$1}.png"
      en = $1

      if ! @opts[:tag] && ! @opts[:tag].is_a?(Proc)
        if @opts[:as_liquid_asset]
          then EmojiLiquidTag % [ep, en]
          else EmojiTag % [
            ep, en
          ]
        end
      else
        @opts[:tag].call(
          ep, en
        )
      end
    end)
  end
end
