require "content/pipeline"

module Content::Pipeline::Jekyll
  class Converter < Jekyll::Converter
    require "content/pipeline/filters/markdown"
    require "content/pipeline/filters/code"
    require "content/pipeline/filters/gemoji"
    require "content/pipeline/filters/blockquote"
    require "content/pipeline/filters/links"

    DefaultFilters = [
      Content::Pipeline::Filters::Markdown,

      Content::Pipeline::Filters::Code,
      Content::Pipeline::Filters::Gemoji,
      Content::Pipeline::Filters::Blockquote,
      Content::Pipeline::Filters::Links
    ]

    def convert(data)
      return Content::Pipeline.new(DefaultFilters).filter(data, {
        :links  => {
          :host => @config["url"].gsub(
            %r!https?://!, ""
          )
        }
      })
    end

    def matches(ext)
      ext =~ \
        /\.(md|markdown)\Z/
    end

    def output_ext(ext)
      ".html"
    end
  end
end
